#include <stdio.h>
#include <math.h>
#include "common.h"
#include "complex.h"
#include "vector.h"

void
vec_xgnuputfiles(const struct vector_t *vp, const char *filename,const int);

void
vec_gnuput(const struct vector_t *vp, const char *fname, const int many)
{
	int	i;
	FILE	*fp;

	int	*ip;
	float	*sp;
	double	*dp;
	struct simplex_t *cp;
	struct duplex_t *zp;

	vec_check(__F, vp, -1);

	fp = fopen(fname, "w");
	if (fp == NULL)
		fatal("%s(\"%s\", \"%s\"): can not fopen", __F,
			vp->vt_name, fname);
	if (vp->vt_size % many)
		fatal("%s(): not divisible", __F);

	switch (vp->vt_type) {
		case SP_INTEGER:
			ip = ivec(vp);
			for (i=0;i<vp->vt_size;i++) {
				fprintf(fp, "%10d\n", ip[i]);
				if (((i+1) % many) == 0)
					fprintf(fp, "\n");
			}
			break;
		case SP_SINGLE:
			sp = svec(vp);
			for (i=0;i<vp->vt_size;i++) {
				fprintf(fp, "%12.5e\n", sp[i]);
				if (((i+1) % many) == 0)
					fprintf(fp, "\n");
			}
			break;
		case SP_DOUBLE:
			dp = dvec(vp);
			for (i=0;i<vp->vt_size;i++) {
				fprintf(fp, "%12.5e\n", dp[i]);
				if (((i+1) % many) == 0)
					fprintf(fp, "\n");
			}
			break;
		case SP_SIMPLEX:
			cp = cvec(vp);
			for (i=0;i<vp->vt_size;i++) {
				fprintf(fp, "%12.5e %12.5e\n",
					cp[i].real, cp[i].imag);
				if (((i+1) % many) == 0)
					fprintf(fp, "\n");
			}
			break;
		case SP_DUPLEX:
			zp = zvec(vp);
			for (i=0;i<vp->vt_size;i++) {
				fprintf(fp, "%12.5e %12.5e\n",
					zp[i].real, zp[i].imag);
				if (((i+1) % many) == 0)
					fprintf(fp, "\n");
			}
			break;
	}
	fclose(fp);
	return;
}

void
vec_gnuputfiles(const struct vector_t *vp, const char *fname, const int many)
{
	vec_check(__F, vp, -1);
	switch (vp->vt_type) {
		case SP_INTEGER:
			vec_gnuput(vp, fname, many);
			break;
		case SP_SINGLE:
			vec_gnuput(vp, fname, many);
			break;
		case SP_DOUBLE:
			vec_gnuput(vp, fname, many);
			break;
		case SP_SIMPLEX:
			vec_xgnuputfiles(vp, fname, many);
			break;
		case SP_DUPLEX:
			vec_xgnuputfiles(vp, fname, many);
			break;
		default:
			fatal("%s():I dont understand your type", __F);
			break;
	}
	return;
}

/*
 * A private function of vec_printatfiles()
 * DONT call this function directly
 */
void
vec_xgnuputfiles(const struct vector_t *vp, const char *filename,const int many)
{
	FILE	*fp[3];
	char	fname[3][132];
	char	*tails[3]={"real", "imag", "inten"};
	int	i;
	struct simplex_t *cp;
	struct duplex_t *zp;

	if (vp->vt_size % many)
		fatal("%s(): not divisible", __F);

	for (i=0;i<3;i++) {
		sprintf(fname[i], "%s.%s", filename, tails[i]);
		fp[i] = fopen(fname[i], "w");
		if (fp[i] == NULL)
			fatal("%s(\"%s\"): can not open", __F, fname[i]);
	}

	switch (vp->vt_type) {
		case SP_SIMPLEX:
			cp = cvec(vp);
			for (i=0; i < vp->vt_size; i++) {
				fprintf(fp[0], "%10.6e\n", cp[i].real);
				fprintf(fp[1], "%10.6e\n", cp[i].imag);
				fprintf(fp[2], "%10.6e\n", cp_absolute(&cp[i]));
				if (((i+1) % many) == 0) {
					fprintf(fp[0], "\n");
					fprintf(fp[1], "\n");
					fprintf(fp[2], "\n");
				}
			}
			break;
		case SP_DUPLEX:
			zp = zvec(vp);
			for (i=0;i<vp->vt_size;i++) {
				fprintf(fp[0], "%10.6e\n", zp[i].real);
				fprintf(fp[1], "%10.6e\n", zp[i].imag);
				fprintf(fp[2], "%10.6e\n", zp_absolute(&zp[i]));
				if (((i+1) % many) == 0) {
					fprintf(fp[0], "\n");
					fprintf(fp[1], "\n");
					fprintf(fp[2], "\n");
				}
			}
	}

	for (i=0;i<3;i++)
		fclose(fp[i]);
	
	return;
}
